# Generated by Django 3.2.23 on 2024-02-06 11:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_twined", "0013_alter_serviceusageevent_question"),
    ]

    operations = [
        migrations.AddField(
            model_name="question",
            name="status",
            field=models.IntegerField(
                choices=[
                    (-100, "No status"),
                    (-3, "Failed (invalid inputs)"),
                    (-2, "Failed (timeout)"),
                    (-1, "Failed (error)"),
                    (0, "In progress"),
                    (1, "Complete"),
                ],
                default=-100,
            ),
        ),
    ]
